#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <stdio.h>
#include <math.h>
#include "simage.h"

float viewDistance = 15.0;
float viewRotY = 0.0;
float viewRotX = 0.0;
float viewCameraX = 0.0;
float viewCameraY = 0.0;
float prevTime = 0.0;

float floorQuad[4][3] =
{
    { -100.0f, 0.0f, -100.0f },
    { -100.0f, 0.0f,  100.0f },
    {  100.0f, 0.0f,  100.0f },
    {  100.0f, 0.0f, -100.0f },
};

GLuint textureID0 = 0;
GLuint textureID1 = 0;
GLuint textureID2 = 0;

GLfloat lightPosition[] = {-2, 10, 2, 1};

GLfloat lightAmbient[] = {0.2, 0.2, 0.2, 1.0};
GLfloat lightDiffuse[] = {1, 1, 1, 1};
GLfloat lightSpecular[] = {1, 1, 1, 0.25};

GLfloat white[] = {1, 1, 1, 1};
GLfloat green[] = {0, 1, 0, 1};
GLfloat black[] = {0, 0, 0, 1};
GLfloat red[] = {1, 0, 0, 1};

float ballPos[3] = {0, 5, -5};
GLUquadricObj * quadric;

void initTexture(char *filename, GLuint& textureID)
{
	unsigned char *imageBuffer;
	int imageWidth = 0, imageHeight = 0, numComponents = 0;
	imageBuffer = simage_read_image(filename, &imageWidth, &imageHeight, &numComponents);

	if (imageBuffer != NULL)
	{
		glGenTextures(1, &textureID);
		glBindTexture(GL_TEXTURE_2D, textureID);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		if (numComponents == 4)
		{
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, imageWidth, imageHeight,
					 0, GL_RGBA, GL_UNSIGNED_BYTE, imageBuffer);
		}
		else
		{
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, imageWidth, imageHeight,
					 0, GL_RGB, GL_UNSIGNED_BYTE, imageBuffer);
		}
		glBindTexture(GL_TEXTURE_2D, 0);
	}
}

void drawAlphaTexSquare()
{
	glColor4f(1.0f, 1.0f, 1.0f, 0.5f);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, textureID0);

	glBegin(GL_QUADS);
	glTexCoord2f(0, 0);
	glVertex3f(-10, 0, 0);
	glTexCoord2f(1, 0);
	glVertex3f(10, 0, 0);
	glTexCoord2f(1, 1);
	glVertex3f(10, 10, 0);
	glTexCoord2f(0, 1);
	glVertex3f(-10, 10, 0);
	glEnd();

	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
}

void drawTexFloor()
{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, textureID2);

	glBegin(GL_QUADS);
	glTexCoord2f(0, 0);
	glVertex3fv(floorQuad[0]);
	glTexCoord2f(1, 0);
	glVertex3fv(floorQuad[1]);
	glTexCoord2f(1, 1);
	glVertex3fv(floorQuad[2]);
	glTexCoord2f(0, 1);
	glVertex3fv(floorQuad[3]);
	glEnd();

	glDisable(GL_TEXTURE_2D);
}

void display()
{
	glClearColor(0.5, 0.7, 1, 0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// light
	glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60, 1, 0.1, 100);
	glMatrixMode(GL_MODELVIEW);

	glLoadIdentity();
	glTranslatef(0.0, 0.0, -viewDistance);
	glRotatef(viewRotX, 1.0, 0.0, 0.0);
	glRotatef(viewRotY, 0.0, 1.0, 0.0);

	//glMaterialfv(GL_FRONT, GL_DIFFUSE, green);
	glPushMatrix();
	glTranslatef(0.0, -1.5, 0.0);
	drawTexFloor();
	glPopMatrix();

	//glMaterialfv(GL_FRONT, GL_DIFFUSE, white);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, textureID1);
	glPushMatrix();
	glTranslatef(ballPos[0], ballPos[1], ballPos[2]);
	//gluSphere(quadric, 1.0, 32, 16);
	glutSolidTeapot(1.5);
	glPopMatrix();
	glDisable(GL_TEXTURE_2D);

	drawAlphaTexSquare(); //textureID0

	glutSwapBuffers();
}

void keyboard(unsigned char key, int x, int y)
{
	if (key == 27)
		exit(0);
	else if (key == ' ')
	{
	}
	else if (key == 'a')
	{
		viewDistance -= 1;
	}
	else if (key == 's')
	{
		viewDistance += 1;
	}
	glutPostRedisplay();
}

void specialkey(int key, int x, int y)
{
	if (key == GLUT_KEY_LEFT)
	{
		viewCameraY += 90;
	}
	else if (key == GLUT_KEY_RIGHT)
	{
		viewCameraY -= 90;
	}
	else if (key == GLUT_KEY_UP)
	{
		viewCameraX += 90;
	}
	else if (key == GLUT_KEY_DOWN)
	{
		viewCameraX -= 90;
	}
}

void specialkeyUp(int key, int x, int y)
{
	if (key == GLUT_KEY_LEFT)
	{
		viewCameraY -= 90;
	}
	else if (key == GLUT_KEY_RIGHT)
	{
		viewCameraY += 90;
	}
	else if (key == GLUT_KEY_UP)
	{
		viewCameraX -= 90;
	}
	else if (key == GLUT_KEY_DOWN)
	{
		viewCameraX += 90;
	}
}

void update()
{
	static int currentTime, deltaTime, prevTime = 0;
	currentTime = glutGet(GLUT_ELAPSED_TIME);
	deltaTime = currentTime - prevTime;
	prevTime = currentTime;
	viewRotX += viewCameraX * (float) (deltaTime) * 0.001;
	viewRotY += viewCameraY * (float) (deltaTime) * 0.001;
	ballPos[0] = sinf((float) currentTime * 0.001) * 5;
	ballPos[2] = cosf((float) currentTime * 0.001) * 5;
	ballPos[1] = fabs(sinf((float) currentTime * 0.001)) * 5;

	glutPostRedisplay();
}

int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(400, 400);
	glutInitWindowPosition(0,0);
	glutCreateWindow(argv[0]);
	glutDisplayFunc(display);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(specialkey);
	glutSpecialUpFunc(specialkeyUp);
	glutIgnoreKeyRepeat(1);
	glutIdleFunc(update);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_DEPTH_TEST);
	quadric = gluNewQuadric();
	initTexture("tree.tga", textureID0);
	initTexture("slime2.jpg", textureID1);
	initTexture("grass1.jpg", textureID2);
	glutMainLoop();
	return 0;
}